unit Decl;

interface

uses
{$ifdef DEBUG_MODE}
    Inifiles, Debug;
{$else}
    Inifiles;
{$endif}


{declarations used in forms etc}
const
     MAX_LIST_ENTRIES = 20;
     MAX_DOS_COMMAND = 140;

     UNIQUE_LENGTH = 11;

     {Items used to process the .ini file}
     LIT_PGP_EXE = 'PGP.EXE';
     LIT_OPTIONS = 'Options';
     LIT_ASCII_OUT = 'OutPutAsASCII';
     LIT_PGPWIPESOURCE = 'PGPWipeSource';
     LIT_CONVWIPESOURCE = 'ConvWipeSource';
     LIT_CONVWIPECIPHER = 'ConvWipeCipherText';
     LIT_SIGN = 'SignOutput';
     LIT_RECOVER = 'RecoverFileName';
     LIT_PGPPATH = 'PGPExecutablePath';
     LIT_HISTORY = 'FileHistory';
     LIT_EXPERT = 'ExpertMode';
     LIT_DEC = 'Recently Decoded Files';
     LIT_ENC = 'Recently Encoded Files';
     LIT_NO = 'No';
     LIT_YES = 'Yes';
     LIT_PUBLIC_KEY_ID = 'pub';
     LIT_SECRET_KEY_ID = 'sec';
     LIT_PUBLIC_KEYS = 'Public Key IDs';
     LIT_SECRET_KEYS = 'Secret Key IDs';
     LIT_KEYRINGS = 'Keyrings';
     LIT_PUBRINGDATE = 'PublicRingDate';
     LIT_SECRINGDATE = 'SecretRingDate';
     LIT_TIMEZONE = 'TimeZone';
     LIT_TEMPPATH = 'TempPath';

     // Error messages
     LIT_MSG1 = 'This is message 1';
     LIT_MSG2 = 'The location of the PGP Executable is not set, please set it';
     LIT_MSG3 = 'Set Environment variable TEMP';
     LIT_MSG4 = 'Set Environment variable TZ';
     LIT_MSG5 = 'Set Environment variable PGPPath';
     LIT_MSG6 = 'The DOS environment is incorrectly set. WinPGP cannot start';
     LIT_MSG7 = 'Error opening output file. Dos Error: ';
     LIT_MSG8 = 'The selected file is not a WinPGP file';
     LIT_MSG9 = 'The passphrase is not correct';
     LIT_MSG10 = 'A file with the same name as the output file already exists. Overwrite?';
     LIT_MSG11 = 'Error opening input file. Dos Error: ';
     LIT_MSG12 = 'An Error occurred shelling out to PGP.';
     LIT_MSG13 = 'The PGP executable is not in the directory specified in the options dialog, please check';
     LIT_MSG14 = 'The PGP executable is not in the directory specified.';

var
     FileInHandle: Integer;
     FileOutHandle: Integer;
     WinPGPIni: TIniFile;
     {$ifdef DEBUG_MODE}
     frmDebug: TfrmDebug;
     {$endif}

function GetEnvironmentVariable(Variable: String; var Parameter: String): Boolean;
function CheckFileTimeChange(FilePathToCheck: String; IniLiteral: String): Boolean;
function  GetEncryptOptions: string;
function  GetDecryptOptions: string;
procedure CleanTempFiles;
procedure Initialise;
procedure ExecutePGP(Command: string; DisplayMode: Integer);
procedure WorkListHeader;
procedure LogEvent(EventToLog: string);
function OpenInputFile(SourceFileName: string): Boolean;
function OpenOutputFile(TargetFileName: string): Boolean;
function CheckDuplicateName(var InputName: string): Boolean;
procedure WriteIniOption(Name: String; Value: Boolean);
procedure KillFile(FileToKill: string);

type
    TFileInfo = record
    Name: string;
    NameLength: Integer;
    FileLength: LongInt;
    Date: TDateTime;
end;

implementation

uses
     SysUtils, WinProcs, WinTypes, Dialogs, Main, ExtCtrls, Pass;

function GetEnvironmentVariable(Variable: String; var Parameter: String): Boolean;
var
     EnvPointer: PChar;
     WorkString1: string;
     WorkString2: string;
     WorkString3: string;
     DelimPos: Integer;
     Length: Integer;
begin
     {$ifdef 32BIT}
     EnvPointer := GetEnvironmentStrings;
     {$else}
     EnvPointer := GetDOSEnvironment;
     {$endif}

     {Set the default result}
     Result := False;
     repeat
     begin
          WorkString1 := StrPas(EnvPointer);
          DelimPos := Pos('=',WorkString1);
          WorkString2 := Copy(WorkString1,0,DelimPos-1);
          WorkString3 := Copy(WorkString1,DelimPos+1,256);
          if LowerCase(Workstring2) = Variable then
          begin
               Result := True;
               Parameter := WorkString3;
          end;
          Length := StrLen(EnvPointer);
          EnvPointer := EnvPointer + Length + 1;
     end;
     until Length = 0;
end;

function CheckFileTimeChange(FilePathToCheck: String; IniLiteral: String): Boolean;
var
     WorkDateTime : TDateTime;
     RingHandle: Integer;
begin
     WorkDateTime := StrToDateTime(WinPGPIni.ReadString(LIT_KEYRINGS, IniLiteral, DateTimeToStr(Now)));
     RingHandle := FileOpen(FilePathToCheck, 0);

     {is there a file there?}
     if RingHandle = -1 then
     begin
          Result := False;
          Exit;
     end;

     If WorkDateTime <> FileDateToDateTime(FileGetdate(RingHandle)) then
     begin
          Result:= True;
     end
     else
     begin
          Result := False;
     end;
     FileClose(RingHandle);
end;

procedure KillFile(FileToKill: string);
var
     ConvString: PChar;
begin
     {$ifdef 32BIT}
     GetMem(ConvString,255);
     StrPCopy(ConvString,FileToKill);
     DeleteFile(ConvString);
     Dispose(ConvString);
     {$else}
     DeleteFile(FileToKill);
     {$endif}
end;

procedure WriteIniOption(Name: String; Value: Boolean);
begin
     if Value then
          WinPGPIni.WriteString(LIT_OPTIONS,Name,LIT_YES)
     else
          WinPGPIni.WriteString(LIT_OPTIONS,Name,LIT_NO);
end;

function CheckDuplicateName(var InputName: string): Boolean;
begin
     {Win31 only}
     {Check if the target name already exists}
     Result := True;
     if FileExists(InputName) then
     begin
          {ask for an alternative or overwrite}
          case MessageDlg(LIT_MSG10,mtConfirmation,[mbYes,mbNo,mbCancel],10) of
                idYes:
                      begin
                           Result := True;
                           Exit;
                      end;
                idNo:
                      begin
                           frmWinPGP.dlgSave.FileName := InputName;
                           if frmWinPGP.dlgSave.Execute then
                           begin
                                InputName := frmWinPGP.dlgSave.FileName;
                                Result := True;
                                Exit;
                           end else begin
                                Result := False;
                                Exit;
                           end;

                      end;
                idCancel:
                      begin
                           Result := False;
                           Exit;
                      end;
          end;
     end;
end;

function OpenOutputFile(TargetFileName: string): Boolean;
begin
     {open the output filehandle}
     FileOutHandle := FileCreate(TargetFileName);

     {Check for success opening}
     if FileOutHandle < 0 then
     begin
          {Error opening input file}
          {Todo - new text?}
          MessageDlg(LIT_MSG7 + IntToStr(FileOutHandle),mtError,[mbOK,mbHelp],7);
          Result := False;
          Exit;
     end;
     Result := True;
end;

function OpenInputFile(SourceFileName: string): Boolean;
begin
     {set the default return code}
     Result := True;

     {open the input filehandle}
     FileInHandle := FileOpen(SourceFileName,0);

     {Check for success opening}
     if FileInHandle < 0 then
     begin
          {Error opening input file}
          MessageDlg(LIT_MSG11 + IntToStr(FileInHandle),mtError,[mbOK,mbHelp],11);
          Result := False;
          Exit;
     end;
end;

procedure WorkListHeader;
begin
     LogEvent('WorkListHeader');
     with frmWinPGP.WorkList do
     begin
          Clear;
          Add ('Set TZ='+ WinPGPIni.ReadString(LIT_OPTIONS,LIT_TIMEZONE,''));
          Add ('Set PGPPath='+PGPPath);
          Add('c:');
          Add('cd '+ PGPPath);
          Add('cls');
          Add('@echo WinPGP Pre-Release - Copyright Ian Thomas Sparkes 1997');
     end;
end;

procedure LogEvent(EventToLog: string);
begin
     {$ifdef DEBUG_MODE}
     if frmDebug = nil then
          frmDebug := TfrmDebug.Create(frmWinPGP);
     frmDebug.Show;
     frmDebug.memDebug.Lines.Add(EventToLog);
     if frmDebug.memDebug.Lines.Count > 32000 then
          frmDebug.memDebug.Lines.Clear;
     {$endif}
end;

procedure CleanTempFiles;
begin
     LogEvent('CleanTempFiles');
     KillFile(PGPPath + '\clip.in');
     KillFile(PGPPath + '\clip.out');
     KillFile(PGPPath + '\clip.');
     KillFile(PGPPath + '\pubkeys.txt');
     KillFile(PGPPath + '\seckeys.txt');
     KillFile(PGPPath + '\pubring.asc');
     KillFile(PGPPath + '\secring.asc');
     KillFile(PGPPath + '\extract.bat');
     KillFile(PGPPath + '\decrypt.bat');
     KillFile(PGPPath + '\encrypt.bat');
     KillFile(PGPPath + '\keyop.bat');
end;

Procedure Initialise;
begin
     LogEvent('Initialise');
     {Open the Inifile}
     WinPGPIni := TIniFile.Create('WinPGP.Ini');
     {GetDir(0, Path);}
end;

function GetDecryptOptions: string;
begin
     Result := '';
     if (WinPGPIni.ReadString(LIT_OPTIONS,LIT_RECOVER,LIT_NO) = LIT_YES) then
          Result := Result + '-p';
end;

function GetEncryptOptions: string;
begin
     Result := '';
     if (WinPGPIni.ReadString(LIT_OPTIONS,LIT_ASCII_OUT,LIT_NO) = LIT_YES) then
          Result := Result + 'a';
     if (WinPGPIni.ReadString(LIT_OPTIONS,LIT_PGPWIPESOURCE,LIT_NO) = LIT_YES) then
          Result := Result + 'w';
     if (WinPGPIni.ReadString(LIT_OPTIONS,LIT_SIGN,LIT_NO) = LIT_YES) then
          Result := Result + 's';
end;

procedure ExecutePGP(Command: string; DisplayMode: Integer);
var
     WorkPString: PChar;
     Retcode: Word;
     FileHandle: Integer;
begin
     LogEvent('ExecutePGP Exec: ' + Command);
     {$ifdef DEBUG_MODE}
     DisplayMode := SW_SHOW;
     {$endif}

     frmWinPGP.pnlStatus.Caption := 'PGP Working';
     frmWinPGP.pnlStatus.Repaint;

     {create flag file to control return}
     KillFile(PGPPath + '\flag.fil');
     FileHandle := FileCreate(PGPPath + '\flag.fil');
     FileClose(FileHandle);

     {complete the batch file}
     with frmWinPGP.WorkList do
     begin
          {$ifdef DEBUG_MODE}
          Add('Pause');
          {$endif}
          Add('del ' + PGPPath + '\flag.fil');

          {save it to the correct path}
          SaveToFile(PGPPath + '\' + Command + '.bat');
     end;

     WorkPString := StrAlloc(256);
     StrPCopy(WorkPString, PGPPath + '\' + Command + '.bat');

     Retcode := WinExec(WorkPString, DisplayMode);

     If RetCode < 32 then
     begin
          {Error}
          MessageDlg(LIT_MSG12,mtError,[mbOK],12);
          Exit;
     end
     else
     begin
          {wait for the command to execute}
          while FileExists(PGPPath  + '\Flag.fil') do
          begin
               Yield;
          end;
     end;

     StrDispose(WorkPString);
     frmWinPGP.pnlStatus.Caption := 'PGP Idle';
end;

end.
