unit Main;

interface


uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, ExtCtrls, ShellAPI, Buttons, Gauges;

type
  TfrmWinPGP = class(TForm)
    lstRecipients: TListBox;
    lstSignatures: TListBox;
    mnuMenu: TMainMenu;
    mnhFile: TMenuItem;
    mnpExit: TMenuItem;
    mnhKey: TMenuItem;
    mnhEncrypt: TMenuItem;
    mnpEClipboard: TMenuItem;
    mnpDecrypt: TMenuItem;
    mnpDClipboard: TMenuItem;
    mnpEFile: TMenuItem;
    dlgEncrypt: TOpenDialog;
    mnpDFile: TMenuItem;
    dlgDecrypt: TOpenDialog;
    mnhOptions: TMenuItem;
    mnpCreateKey: TMenuItem;
    mnpAddKeysFile: TMenuItem;
    lblRecipient: TLabel;
    lblSignature: TLabel;
    dlgKey: TOpenDialog;
    pnlStatus: TPanel;
    Memo1: TMemo;
    mnpPublicKey: TMenuItem;
    mnpCopyKeyP: TMenuItem;
    mnpAddKeyP: TMenuItem;
    mnpDelKeyP: TMenuItem;
    mnpSecretKey: TMenuItem;
    mnpFingerP: TMenuItem;
    mnpDelKeyS: TMenuItem;
    mnpEditPass: TMenuItem;
    mnhHelp: TMenuItem;
    mnpSeparator: TMenuItem;
    mnpAbout: TMenuItem;
    mnpHelp: TMenuItem;
    mnpCopyKeyS: TMenuItem;
    mnpAddKeyS: TMenuItem;
    dlgSave: TSaveDialog;
    procedure mnpExitClick(Sender: TObject);
    procedure mnpRefreshClick(Sender: TObject);
    procedure mnpEClipboardClick(Sender: TObject);
    procedure mnpEFileClick(Sender: TObject);
    procedure mnpDFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure mnpAddKeysFileClick(Sender: TObject);
    procedure mnpDClipboardClick(Sender: TObject);
    procedure mnhOptionsClick(Sender: TObject);
    procedure lstRecipientsClick(Sender: TObject);
    procedure mnpCreateKeyClick(Sender: TObject);
    procedure mnpCopyKeyPClick(Sender: TObject);
    procedure mnpEditPassClick(Sender: TObject);
    procedure mnpDelKeyPClick(Sender: TObject);
    procedure mnpFingerPClick(Sender: TObject);
    procedure mnpDelKeySClick(Sender: TObject);
    procedure mnpAddKeyPClick(Sender: TObject);
    procedure mnpAboutClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnpHelpClick(Sender: TObject);
    procedure mnpAddKeySClick(Sender: TObject);
    procedure mnpCopyKeySClick(Sender: TObject);
  private
    WorkListTemp: string;
    procedure ExtractKeys;
    procedure LoadLists;
    procedure SetMenu;
    procedure AppMessage(var Msg:TMsg; var Handled:Boolean);
    procedure AskEncDec(FileName: string);
    procedure EncryptHistory(EncryptFilename: string);
    procedure DecryptHistory(DecryptFilename: string);
  public
    WorkList: TStringList;
    SessionPassWord: string;
  end;

procedure DecryptFile(DecryptFileName: string);
procedure EncryptFile(EncryptFileName: string);
procedure ConventionalDecrypt(DecryptFileName: string);
procedure ConventionalEncrypt(EncryptFileName: string);

var
  frmWinPGP: TfrmWinPGP;
  PGPPath: String;
implementation

{$R *.DFM}

uses
    Options, Decl, About, EncDec;

function BLOWINIT(Key: PChar; Length: Integer; Context: PChar): Integer; far; external 'WPGP';
function BLOWENC(Data: PChar; Blocks: Integer; Context: PChar): Integer; far; external 'WPGP';
function BLOWDEC(Data: PChar; Blocks: Integer; Context: PChar): Integer; far; external 'WPGP';

procedure TfrmWinPGP.EncryptHistory(EncryptFilename: string);
begin
     {Add the file to the history list}
     with dlgEncrypt.HistoryList do
     begin
          Insert(0, LowerCase(EncryptFileName));
          if count > MAX_LIST_ENTRIES then
          begin
               Delete(count-1);
          end;
     end;
end;

procedure TfrmWinPGP.DecryptHistory(DecryptFilename: string);
begin
     with dlgDecrypt.HistoryList do
     begin
          Insert(0, LowerCase(DecryptFileName));
          if count > MAX_LIST_ENTRIES then
          begin
               Delete(count-1);
          end;
     end;
end;

procedure TfrmWinPGP.ExtractKeys;
var
     ItemPointer: Integer;
     WorkString: String;
     RingHandle: Integer;
     WorkDateTime: TDateTime;
begin
     LogEvent('TfrmWinPGP.ExtractKeys');

     {Delete the output files}
     CleanTempFiles;

     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -kv ' + PGPPath + '\pubring.pgp > ' + PGPPath + '\pubkeys.txt';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     WorkListTemp := PGPPath + '\pgp -kv ' + PGPPath + '\secring.pgp > ' + PGPPath + '\seckeys.txt';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);

     {Shell out to PGP}
     ExecutePGP('Extract',SW_SHOWMINIMIZED);

     {Clean up the list of public keys}
     WorkList.LoadFromFile(PGPPath  + '\pubkeys.txt');
     ItemPointer := 0;
     While ItemPointer <= WorkList.Count -1 do
     begin
          WorkString := Copy(WorkList[ItemPointer], 0, 3);
          if WorkString <> LIT_PUBLIC_KEY_ID then
          begin
               WorkList.Delete(ItemPointer);
          end
          else
          begin
               WorkList[Itempointer] := Copy(WorkList[Itempointer],30,Length(WorkList[Itempointer])-29);
               Inc (ItemPointer);
          end;
     end;

     {Write the public list to the inifile}
     WorkList.Sorted := True;
     WinPGPIni.EraseSection(LIT_PUBLIC_KEYS);
     For ItemPointer := 0 to WorkList.Count -1 do
     begin
          WinPGPIni.WriteString(LIT_PUBLIC_KEYS,'PK'+IntToStr(ItemPointer), WorkList[ItemPointer]);
     end;
     WorkList.Sorted := False;

     {Clean up the list of secret keys}
     WorkList.LoadFromFile(PGPPath  + '\seckeys.txt');
     ItemPointer := 0;
     While ItemPointer <= WorkList.Count -1 do
     begin
          WorkString := Copy(WorkList[ItemPointer], 0, 3);
          if WorkString <> LIT_SECRET_KEY_ID then
          begin
               WorkList.Delete(ItemPointer);
          end
          else
          begin
               WorkList[Itempointer] := Copy(WorkList[Itempointer],30,Length(WorkList[Itempointer])-29);
               Inc (ItemPointer);
          end;
     end;

     {Write the secret list to the inifile}
     WinPGPIni.EraseSection(LIT_SECRET_KEYS);
     WorkList.Sorted := True;
     For ItemPointer := 0 to WorkList.Count -1 do
     begin
          WinPGPIni.WriteString(LIT_SECRET_KEYS,'SK'+IntToStr(ItemPointer), WorkList[ItemPointer]);
     end;
     WorkList.Sorted := False;

     {Store the timestamps of the files in the inifile}
     RingHandle := FileOpen(PGPPath  + '\pubring.pgp', 0);
     WorkDateTime := FileDateToDateTime(FileGetdate(RingHandle));
     WinPGPIni.WriteString(LIT_KEYRINGS, LIT_PUBRINGDATE, DateTimeToStr(WorkDateTime));
     FileClose(RingHandle);

     RingHandle := FileOpen(PGPPath  + '\secring.pgp', 0);
     WorkDateTime := FileDateToDateTime(FileGetdate(RingHandle));
     WinPGPIni.WriteString(LIT_KEYRINGS, LIT_SECRINGDATE, DateTimeToStr(WorkDateTime));
     FileClose(RingHandle);
     LoadLists;
end;

procedure TfrmWinPGP.mnpExitClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpExitClick');

     Close;
end;

procedure TfrmWinPGP.mnpRefreshClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpRefreshClick');

     ExtractKeys;
end;

procedure TfrmWinPGP.mnpEClipboardClick(Sender: TObject);
var
     i: Integer;
begin
     LogEvent('TfrmWinPGP.mnpEClipboardClick');

     {Delete the output files}
     CleanTempFiles;

     Memo1.Clear;
     Memo1.PasteFromClipboard;
     Memo1.Lines.SaveToFile(PGPPath  + '\clip.in');

     {encrypt the clipboard}
     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -ea ' + PGPPath  + '\clip.in ';
     for i := 0 to lstRecipients.Items.Count -1 do
     begin
          if lstRecipients.Selected[i] then
          begin
               if (Length(WorkListTemp) + Length(lstRecipients.Items[i])) < MAX_DOS_COMMAND then
               begin
                    WorkListTemp := WorkListTemp + ' "' +
                         Copy(lstRecipients.Items[i],0,UNIQUE_LENGTH) + '"';
               end;
          end;
     end;
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     WorkList.Add('Ren clip.asc clip.out');
     ExecutePGP('Encrypt',SW_SHOW);

     Memo1.Clear;
     try
     begin
          Memo1.Lines.LoadFromFile(PGPPath + '\clip.out');
          Memo1.SelectAll;
          Memo1.CopyToClipBoard;
     end;
     except
     end;
     CleanTempFiles;
end;

procedure TfrmWinPGP.mnpEFileClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpEFileClick');

     if dlgEncrypt.Execute then
     begin
          EncryptFile(dlgEncrypt.FileName);
     end;
end;

procedure TfrmWinPGP.mnpDFileClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpDFileClick');

     if dlgDecrypt.Execute then
     begin
          DecryptFile(dlgDecrypt.FileName);
     end;
end;

procedure EncryptFile(EncryptFileName: string);
var
     i: Integer;
begin
     LogEvent('EncryptFile');

     with frmWinPGP do
     begin
          EncryptHistory(EncryptFileName);
          WorkList.Clear;
          WorkList.Add('cls');
          WorkList.Add('@echo WinPGP Pre-Release - Copyright Ian Thomas Sparkes 1997');
          WorkListTemp := PGPPath + '\pgp -e' + GetEncryptOptions + ' ' + EncryptFileName;
          for i := 0 to lstRecipients.Items.Count -1 do
          begin
               if lstRecipients.Selected[i] then
               begin
                    if (Length(WorkListTemp) + Length(lstRecipients.Items[i])) < MAX_DOS_COMMAND then
                    begin
                         WorkListTemp := WorkListTemp + ' "' +
                              Copy(lstRecipients.Items[i],0,UNIQUE_LENGTH) + '"';
                    end;
               end;
           end;
          WorkList.Add(WorkListTemp);
          LogEvent(WorkListTemp);
          ExecutePGP('Encrypt',SW_SHOW);
     end;
end;

procedure DecryptFile(DecryptFileName: string);
var
     TempFilePath: string;
begin
     LogEvent('DecryptFile');

     with frmWinPGP do
     begin
          DecryptHistory(DecryptFileName);
          frmWinPGP.WorkList.Clear;

          {Change to the right directory}
          TempFilePath := Copy(ExtractFilePath(DecryptFileName), 0, Length(ExtractFilePath(DecryptFileName))-1);
          Worklist.Add(Copy(TempFilePath,0,2));
          Worklist.Add('cd ' + TempFilePath);

          WorkList.Add('cls');

          WorkList.Add('@echo WinPGP Pre-Release - Copyright Ian Thomas Sparkes 1997');
          WorkListTemp := PGPPath + '\pgp '  + GetDecryptOptions + ' ' + DecryptFileName;
          WorkList.Add(WorkListTemp);
          LogEvent(WorkListTemp);
          ExecutePGP('Decrypt',SW_SHOW);
     end;
end;

procedure TfrmWinPGP.FormCreate(Sender: TObject);
var
     ChangedFlag: Boolean;
     ListPointer: Integer;
     WorkString1: string;
begin
     LogEvent('TfrmWinPGP.FormCreate');

     Initialise;

     {clear the session Password}
     frmWinPGP.SessionPassWord := '';

     Application.Title := 'WinPGP Release' + VERSION + ' Build';
     frmWinPGP.Caption := Application.Title;

     PGPPath := WinPGPIni.ReadString(LIT_OPTIONS,LIT_PGPPATH,'');

     {Initialise the work list}
     WorkList := TStringList.Create;
     WorkList.Sorted := False;

     {Check that the setup is correct}
     if (PGPPath ='') then
     begin
          {explain why the options window has appeared}
          MessageDlg(LIT_MSG2,mtWarning,[mbOK,mbHelp],2);
          frmOptions := TfrmOptions.Create(self);
          frmOptions.Showmodal;
          frmOptions.Destroy;
     end
     else
     begin
          {we have been told that we have a path to PGP. Is PGP where we think it is?}
          If not FileExists(PGPPath + '\' + LIT_PGP_EXE) then
          begin
               {explain why the options window has appeared}
               MessageDlg(LIT_MSG13,mtWarning,[mbOK,mbHelp],2);
               frmOptions := TfrmOptions.Create(self);
               frmOptions.Showmodal;
               frmOptions.Destroy;
          end;
     end;

     {if there are command line options, process them}
     {As parameters we can accept filepaths and we will them be
      asked what action we want to perform on them}
     for ListPointer := 1 to ParamCount do
     begin
          WorkString1 := ParamStr(ListPointer);
          AskEncDec(WorkString1);
     end;

     {if we were called dynamically, then end automatically}
     if ParamCount > 0 then
          Halt;

     {init the recipients list}
     lstRecipients.ItemIndex := 0;

     {Set options according to mode - expert mode has extra options}
     mnpAddKeyS.Visible := (WinPGPIni.ReadString(LIT_OPTIONS,LIT_EXPERT,LIT_NO) = LIT_YES);
     mnpCopyKeyS.Visible := (WinPGPIni.ReadString(LIT_OPTIONS,LIT_EXPERT,LIT_NO) = LIT_YES);

     {Read in the filenemes}
     if WinPGPIni.ReadString(LIT_OPTIONS,LIT_HISTORY,LIT_YES) = LIT_YES then
     begin
          WinPGPIni.ReadSection(LIT_ENC,WorkList);
          for ListPointer := 0 to WorkList.Count - 1 do
          begin
               dlgEncrypt.HistoryList.Add(WinPGPIni.ReadString(LIT_ENC,'File' + IntToStr(ListPointer),''));
          end;
          Worklist.Clear;
          WinPGPIni.ReadSection(LIT_DEC,WorkList);
          for ListPointer := 0 to WorkList.Count - 1 do
          begin
               dlgDecrypt.HistoryList.Add(WinPGPIni.ReadString(LIT_DEC,'File' + IntToStr(ListPointer),''));
          end;
     end;

     {Check if there have been Key changes since last time}
     ChangedFlag := CheckFileTimeChange(PGPPath + '\pubring.pgp',LIT_PUBRINGDATE) or
                    CheckFileTimeChange(PGPPath + '\secring.pgp',LIT_SECRINGDATE);

     if ChangedFlag then
     begin
          ExtractKeys;
     end;

     {Register the minimised icon & main form as a drop target}
     Application.OnMessage := frmWinPGP.AppMessage;
     DragAcceptFiles(Application.Handle,TRUE);
     DragAcceptFiles(frmWinPGP.Handle,TRUE);
end;

procedure TfrmWinPGP.AppMessage(var Msg:TMsg; var Handled:Boolean);
var
     FileName: PChar;
     FileIndex: Word;
     FileCount: Word;
begin
     if Msg.Message = WM_DROPFILES then
     begin
          FileName := AllocMem(255);
          FileIndex := 0;
          Dec(FileIndex);
          FileCount := DragQueryFile(Msg.wParam, FileIndex, FileName, 255);
          For FileIndex := 0 to FileCount-1 do
          begin
               DragQueryFile(Msg.wParam, FileIndex, FileName, 255);
               AskEncDec(StrPas(FileName));
          end;
          Handled := True;
     end;
end;

procedure TfrmWinpgp.AskEncDec(FileName: string);
var
     frmEncDec: TfrmEncDec;
begin
     {create the question window to ask if we shoud encrypt or decrypt}
     frmEncDec := TfrmEncDec.Create(Self);
     frmEncDec.Action := '';
     {put the filename in the window}
     frmEncDec.lblFileName.Caption := LowerCase(FileName);
     {put the recipient in the window}
     if lstRecipients.SelCount > 0 then
     begin
          frmEncDec.lblRecipientName.Caption := lstRecipients.Items[lstRecipients.ItemIndex];
          frmEncDec.bbnPGPEnc.Enabled := True;
     end
     else
     begin
          frmEncDec.lblRecipientName.Caption := LIT_MSG1;
          frmEncDec.bbnPGPEnc.Enabled := False;
     end;

     {set the default button}
     frmEncDec.Action := 'ConvEnc';
     if LowerCase(ExtractFileExt(FileName)) = '.pgp' then
          frmEncDec.Action := 'PGPDec';
     if LowerCase(ExtractFileExt(FileName)) = '.cry' then
          frmEncDec.Action := 'ConvDec';

          frmEncDec.ShowModal;
          If frmEncDec.Action = 'PGPEnc' then
               EncryptFile(FileName);
          If frmEncDec.Action = 'PGPDec' then
               DecryptFile(FileName);
          If frmEncDec.Action = 'ConvEnc' then
               ConventionalEncrypt(FileName);
          If frmEncDec.Action = 'ConvDec' then
               ConventionalDecrypt(FileName);
          frmEncDec.Destroy;
end;

procedure TfrmWinPGP.FormShow(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.FormShow');

     LoadLists;
end;

procedure TfrmWinPGP.LoadLists;
var
     ListPointer: Integer;
begin
     LogEvent('TfrmWinPGP.LoadLists');

     {Read the public keys into the pick list}
     lstRecipients.Items.Clear;
     lstSignatures.Items.Clear;
     WorkList.Clear;
     WinPGPIni.ReadSection(LIT_PUBLIC_KEYS,WorkList);
     for ListPointer := 0 to WorkList.Count - 1 do
     begin
          lstRecipients.Items.Add(WinPGPIni.ReadString(LIT_PUBLIC_KEYS, WorkList[ListPointer], 'Error'));
     end;

     {Read the secret keys into the pick list}
     WorkList.Clear;
     WinPGPIni.ReadSection(LIT_SECRET_KEYS,WorkList);
     for ListPointer := 0 to WorkList.Count - 1 do
     begin
          lstSignatures.Items.Add(WinPGPIni.ReadString(LIT_SECRET_KEYS, WorkList[ListPointer], 'Error'));
     end;
     SetMenu;
end;

procedure TfrmWinPGP.mnpAddKeysFileClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpAddKeysFileClick');

     if dlgKey.Execute then
     begin
          WorkList.Clear;
          WorkList.Add('cls');
          WorkList.Add('@echo WinPGP Pre-Release - Copyright Ian Thomas Sparkes 1997');
          WorkListTemp := PGPPath + '\pgp -ka ' + dlgKey.FileName;
          WorkList.Add(WorkListTemp);
          LogEvent(WorkListTemp);
          ExecutePGP('Keyop',SW_SHOW);
     end;
     ExtractKeys;
end;

procedure TfrmWinPGP.mnpDClipboardClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpDClipboardClick');

     {Delete the output files}
     CleanTempFiles;

     Memo1.Clear;
     Memo1.PasteFromClipboard;
     Memo1.Lines.SaveToFile(PGPPath + '\clip.in');

     {encrypt the clipboard}
     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp ' + PGPPath + '\clip.in';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     WorkList.Add('Ren clip. clip.out');
     ExecutePGP('Encrypt',SW_SHOW);

     Memo1.Clear;
     try
     begin
          Memo1.Lines.LoadFromFile(PGPPath + '\clip.out');
          Memo1.SelectAll;
          Memo1.CopyToClipBoard;
     end;
     except
     end;
     CleanTempFiles;
end;

procedure TfrmWinPGP.mnhOptionsClick(Sender: TObject);
var
     frmOptions: TfrmOptions;
begin
     LogEvent('TfrmWinPGP.mnhOptionsClick');

     frmOptions := TfrmOptions.Create(self);
     frmOptions.Showmodal;
     frmOptions.Destroy;
     {In case we have changed things that affect menupoints e.g. signature}
     SetMenu;
end;

procedure TfrmWinPGP.lstRecipientsClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.lstRecipientsClick');

     SetMenu;
end;

procedure TfrmWinPGP.SetMenu;
var
     SignaturesSelectedItems: Integer;
     RecipientsSelectedItems: Integer;
     MenuEnable: Boolean;
begin
     LogEvent('TfrmWinPGP.SetMenu');

     {temp: must transfer the listbox properties to a variable to make them work!}
     RecipientsSelectedItems := lstRecipients.SelCount;
     SignaturesSelectedItems := lstSignatures.ItemIndex;

     if (WinPGPIni.ReadString(LIT_OPTIONS,LIT_SIGN,LIT_NO) = LIT_YES) then
     begin
          MenuEnable := ((RecipientsSelectedItems > 0) and
           (SignaturesSelectedItems <> -1));
          mnpEClipBoard.Enabled := MenuEnable;
          mnpEFile.Enabled := MenuEnable;
     end
     else
     begin
          MenuEnable := (RecipientsSelectedItems > 0);
          mnpEClipBoard.Enabled := MenuEnable;
          mnpEFile.Enabled := MenuEnable;
     end;
     mnpCopyKeyP.Enabled := (RecipientsSelectedItems <> -1);
     mnpDelKeyP.Enabled := (RecipientsSelectedItems <> -1);
     mnpFingerP.Enabled := (RecipientsSelectedItems <> -1);
     mnpSecretKey.Enabled := (SignaturesSelectedItems <> -1);
end;

procedure TfrmWinPGP.mnpCreateKeyClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpCreateKeyClick');

     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -kg ';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOW);
     ExtractKeys;
end;

procedure TfrmWinPGP.mnpCopyKeyPClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpCopyKeyPClick');

     CleanTempFiles;
     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -kxa "' + lstRecipients.Items[lstRecipients.ItemIndex] +
          '" ' + PGPPath + '\pubring.pgp';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);

     ExecutePGP('Keyop',SW_SHOWMINIMIZED);

     Memo1.Lines.LoadFromFile(PGPPath + '\pubring.asc');
     Memo1.Lines.Insert(0,'Public Key for ' + lstRecipients.Items[lstRecipients.ItemIndex]);
     Memo1.SelectAll;
     Memo1.CutToClipboard;
     CleanTempFiles;
end;

procedure TfrmWinPGP.mnpEditPassClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpEditPassClick');

     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -ke "' + lstSignatures.Items[lstSignatures.ItemIndex] + '"';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOW);
end;

procedure TfrmWinPGP.mnpDelKeyPClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpDelKeyPClick');

     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -kr "' + lstRecipients.Items[lstRecipients.ItemIndex] +
          '" ' + PGPPath + '\pubring.pgp';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOW);
     ExtractKeys;
end;

procedure TfrmWinPGP.mnpFingerPClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpFingerPClick');

     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -kvc "' + lstRecipients.Items[lstRecipients.ItemIndex] +
          '" ' + PGPPath + '\pubring.pgp > clip.in';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOWMINIMIZED);
     Memo1.Clear;
     Memo1.Lines.LoadFromFile(PGPPath + '\clip.in');
     Memo1.SelectAll;
     Memo1.CutToClipboard;
     CleanTempFiles;
end;

procedure TfrmWinPGP.mnpDelKeySClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpDelKeySClick');

     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -kr "' + lstSignatures.Items[lstSignatures.ItemIndex] +
          '" ' + PGPPath + '\secring.pgp';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOW);
     ExtractKeys;
end;

procedure TfrmWinPGP.mnpAddKeyPClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpAddKeyPClick');

     Memo1.Clear;
     Memo1.PasteFromClipboard;
     Memo1.Lines.SaveToFile(PGPPath + '\clip.out');
     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -ka ' + PGPPath + '\clip.out ' + PGPPath + '\pubring.pgp';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOW);
     ExtractKeys;
     CleanTempFiles;
end;

procedure TfrmWinPGP.mnpAddKeySClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpAddKeySClick');

     Memo1.Clear;
     Memo1.PasteFromClipboard;
     Memo1.Lines.SaveToFile(PGPPath + '\clip.out');
     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -ka ' + PGPPath + '\clip.out ' + PGPPath + '\secring.pgp';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOW);
     ExtractKeys;
     CleanTempFiles;
end;

procedure TfrmWinPGP.mnpAboutClick(Sender: TObject);
var
     frmAbout: TfrmAbout;
begin
     LogEvent('TfrmWinPGP.mnpAboutClick');

     frmAbout := TfrmAbout.Create(self);
     frmAbout.ShowModal;
end;

procedure TfrmWinPGP.FormDestroy(Sender: TObject);
var
     ListPointer: Integer;
begin
     LogEvent('TfrmWinPGP.FormDestroy');

     if WinPGPIni.ReadString(LIT_OPTIONS,LIT_HISTORY,LIT_YES) = LIT_YES then
     begin
          {Save the key file history}
          WinPGPIni.EraseSection(LIT_ENC);
          WinPGPIni.EraseSection(LIT_DEC);
          for ListPointer := 0 to dlgEncrypt.HistoryList.Count - 1 do
          begin
               WinPGPIni.WriteString(LIT_ENC,'File' + IntToStr(ListPointer),dlgEncrypt.HistoryList[ListPointer]);
          end;
          for ListPointer := 0 to dlgDecrypt.HistoryList.Count - 1 do
          begin
               WinPGPIni.WriteString(LIT_DEC,'File' + IntToStr(ListPointer),dlgDecrypt.HistoryList[ListPointer]);
          end;
     end;
end;

procedure TfrmWinPGP.mnpHelpClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpHelpClick');

     Application.HelpCommand(HELP_CONTENTS,0);
end;

procedure TfrmWinPGP.mnpCopyKeySClick(Sender: TObject);
begin
     LogEvent('TfrmWinPGP.mnpCopyKeySClick');

     CleanTempFiles;
     WorkListHeader;
     WorkListTemp := PGPPath + '\pgp -kxa "' + lstSignatures.Items[lstSignatures.ItemIndex] +
          '" ' + PGPPath + '\secring.asc ' + PGPPath + '\secring.pgp';
     WorkList.Add(WorkListTemp);
     LogEvent(WorkListTemp);
     ExecutePGP('Keyop',SW_SHOW);

     Memo1.Lines.LoadFromFile(PGPPath + '\secring.asc');
     Memo1.Lines.Insert(0,'Secret Key for ' + lstSignatures.Items[lstSignatures.ItemIndex]);
     Memo1.SelectAll;
     Memo1.CutToClipboard;
     CleanTempFiles;
end;

procedure ConventionalEncrypt(EncryptFileName: string);
begin
end;

procedure ConventionalDecrypt(DecryptFileName: string);
begin
end;

end.
